@@ -268,7 +268,12 @@ IOReturn AppleUSBAudioDevice::performPowerStateChange (IOAudioDevicePowerState o
 	if (oldPowerState == kIOAudioDeviceSleep) {
 		debugIOLog ("Waking from sleep - flushing controls to the device.");
 		flushAudioControls ();
+		// [rdar://4234453] Reset the device after waking from sleep just to be safe.
+		FailIf (NULL == mControlInterface, Exit);
+		mControlInterface->GetDevice()->ResetDevice();
+		IOSleep (10);										// Give the device 10 ms to get ready
 	}
+Exit:
 
 	return result;
 }
