@@ -57,6 +57,9 @@ class AppleUSBAudioDevice;
 #define	kChannelDepth_STEREO			2
 
 #define kMinimumFrameOffset				1
+#define kWallTimeExtraPrecision         10000
+#define kRefreshInterval				128
+#define kRefreshCount					8
 
 #define RECORD_NUM_USB_FRAME_LISTS				128
 #define RECORD_NUM_USB_FRAMES_PER_LIST			2
@@ -101,6 +104,7 @@ public:
     static void readHandler (void * object, void * frameListIndex, IOReturn result, IOUSBLowLatencyIsocFrame * pFrames);
     static void writeHandler (void * object, void * parameter, IOReturn result, IOUSBLowLatencyIsocFrame * pFrames);
 	static IOReturn iSubAttachChangeHandler (IOService *target, IOAudioControl *passThruControl, SInt32 oldValue, SInt32 newValue);
+	virtual IOReturn iSubAttachChangeCall (Boolean isiSubCompatible);
 	virtual IOReturn pluginDeviceRequest (IOUSBDevRequest * request, IOUSBCompletion * completion);
 	virtual void pluginSetConfigurationApp (const char * bundleID);
 	virtual void registerPlugin (AppleUSBAudioPlugin * thePlugin);
@@ -111,6 +115,25 @@ protected:
 	volatile UInt32						shouldStop;
 	UInt64								usbFrameToQueueAt;
 	UInt64								nextSynchReadFrame;
+	
+	UInt64 *							mFrameQueuedForList;
+	UInt64								mWallTimePerUSBCycle;
+	UInt64								mReferenceUSBFrame;
+	AbsoluteTime						mReferenceWallTime;
+	UInt64								mNewReferenceUSBFrame;
+	AbsoluteTime						mNewReferenceWallTime;
+	UInt64								mLastUSBFrame;
+	UInt64								mLastWallTime_nanos;
+	#if DEBUGTIMESTAMPS
+		UInt64								mLastTimeStamp_nanos;
+		UInt64								mLastStampDifference;
+		SInt64								mStampDrift;
+		UInt64								mLastWrapFrame;
+	#endif
+	UInt8								mAnchorResetCount;
+	UInt8								mHubSpeed;
+	IOTimerEventSource *				mUpdateTimer;
+	
 	IOUSBLowLatencyIsocFrame *			theUSBIsocFrames;
 	IOUSBIsocFrame						theSampleRateFrame;
 	IOUSBLowLatencyIsocCompletion *		usbCompletion;
@@ -118,7 +141,7 @@ protected:
 	IOUSBInterface *					streamInterface;
 	IOUSBPipe *							thePipe;
 	IOUSBPipe *							theAssociatedPipe;
-	IOSyncer *							signal;
+	IOSyncer *							mSyncer;
 	AppleUSBAudioDevice *				usbAudioDevice;
 	IOBufferMemoryDescriptor *			mUSBBufferDescriptor;
 	IOBufferMemoryDescriptor *			sampleBufferMemoryDescriptor;
@@ -135,7 +158,7 @@ protected:
 	thread_call_t						iSubTeardownThreadCall;
 	thread_call_t						mPluginInitThread;
 	thread_call_t						iSubOpenThreadCall;
-	thread_call_t						firstUSBFrameCompletionThread;
+//	thread_call_t						firstUSBFrameCompletionThread;
 	IOAudioSampleRate					curSampleRate;
 	UInt32								previouslyPreparedBufferOffset;
 	UInt32								safeToEraseTo;
@@ -209,6 +232,7 @@ protected:
 	IOReturn	SetSampleRate (USBAudioConfigObject *usbAudio, UInt32 sampleRate);
 	IOReturn	AddAvailableFormatsFromDevice (USBAudioConfigObject *usbAudio);
 	IOReturn	CheckForAssociatedEndpoint (USBAudioConfigObject *usbAudio);
+	IOReturn	GetDefaultSettings (UInt8 * altSettingID, IOAudioSampleRate * sampleRate);	// added for rdar://3866513 
 
     static bool audioDevicePublished (AppleUSBAudioEngine *audioEngine, void *ref, IOService *newService);
     static bool	iSubEnginePublished (AppleUSBAudioEngine * usbAudioEngineObject, void * refCon, IOService * newService);
@@ -228,6 +252,7 @@ protected:
 	static void iSubTeardown (AppleUSBAudioEngine *usbAudioEngine, thread_call_t iSubTeardownThreadCall);
 	static void iSubOpen (AppleUSBAudioEngine *usbAudioEngineObject);
 	virtual void iSubTeardownConnection (void);
+	bool isiSubCompatibleFormat (const IOAudioStreamFormat *format, IOAudioSampleRate sampleRate);
 
     virtual UInt32 getCurrentSampleFrame (void);
 
@@ -238,9 +263,18 @@ protected:
 	virtual void resetClipPosition (IOAudioStream *audioStream, UInt32 clipSampleFrame);
     virtual IOReturn clipOutputSamples (const void *mixBuf, void *sampleBuf, UInt32 firstSampleFrame, UInt32 numSampleFrames, const IOAudioStreamFormat *streamFormat, IOAudioStream *audioStream);
 	virtual IOReturn convertInputSamples (const void *sampleBuf, void *destBuf, UInt32 firstSampleFrame, UInt32 numSampleFrames, const IOAudioStreamFormat *streamFormat, IOAudioStream *audioStream);
-
+	
     virtual IOReturn performFormatChange (IOAudioStream *audioStream, const IOAudioStreamFormat *newFormat, const IOAudioSampleRate *newSampleRate);
 	void CalculateSamplesPerFrame (UInt32 sampleRate, UInt16 * averageFrameSize, UInt16 * additionalSampleFrameFreq);
+	
+	IOReturn getAnchorFrameAndTimeStamp (UInt64 *frame, AbsoluteTime *time);
+	IOReturn getFrameAndTimeStamp (UInt64 *frame, AbsoluteTime *time);
+	UInt64 jitterFilter (UInt64 invCoefficient, UInt64 prev, UInt64 curr);
+	void updateWallTimePerUSBCycle ();
+	AbsoluteTime generateTimeStamp (UInt32 usbFrameIndex, UInt32 preWrapBytes, UInt32 byteCount);
+	static	void TimerAction (OSObject * owner, IOTimerEventSource * sender);
+	void doTimerAction (IOTimerEventSource * timer);
+
 };
 
 #endif /* _APPLEUSBAUDIOENGINE_H */
