@@ -645,11 +645,10 @@ void AppleiSubEngine::WriteHandler (void * object, void * buffer, IOReturn resul
 	AbsoluteTime					t_now;
 	AbsoluteTime					t_old;
 	UInt64							nanos_del;
-	t_old.lo = self->mLastTime.lo;
-	t_old.hi = self->mLastTime.hi;
+	
+	t_old = self->mLastTime;
 	clock_get_uptime (&t_now);
-	self->mLastTime.lo = t_now.lo;
-	self->mLastTime.hi = t_now.hi;
+	self->mLastTime = t_now;
 	SUB_ABSOLUTETIME (&t_now, &t_old);		// t_now -= t_old;
 	absolutetime_to_nanoseconds (t_now, &nanos_del);
 
